use std::ops::{Index, IndexMut};

use super::header::Header;

#[derive(Debug, Clone)]
pub struct Body(Vec<u8>);

impl Body {
    pub fn body_type(&self) -> u8 {
        self.0[0]
    }

    pub fn raw(&self) -> &[u8] {
        &self.0
    }

    pub fn len(&self) -> usize {
        self.0.len()
    }
}

impl From<&[u8]> for Body {
    fn from(value: &[u8]) -> Self {
        let data = value[(Header::HEADER_LENGTH as usize)..value.len() - 1].to_vec();

        Self(data)
    }
}

impl From<(&[u8], u8)> for Body {
    fn from((data, body_type): (&[u8], u8)) -> Self {
        let mut v = data.to_vec();
        v.insert(0, body_type);

        Self(v)
    }
}

impl Index<usize> for Body {
    type Output = u8;

    fn index(&self, index: usize) -> &Self::Output {
        &self.0[index]
    }
}

impl IndexMut<usize> for Body {
    fn index_mut(&mut self, index: usize) -> &mut Self::Output {
        &mut self.0[index]
    }
}
