use std::ops::Deref;

use super::{body::Body, header::Header, Command, MessageType};

pub struct CommandResponse {
    command: Command,
}

impl CommandResponse {
    pub fn new(msg: &[u8]) -> Self {
        Self {
            command: Command {
                header: Header::from(msg),
                body: Body::from(msg),
            },
        }
    }

    pub fn header(&self) -> &Header {
        &self.command.header
    }

    pub fn body(&self) -> &Body {
        &self.command.body
    }
}

pub struct CommandSubtypeResponse {
    command: CommandResponse,
    pub sub_type: u16,
}

impl CommandSubtypeResponse {
    pub fn new(msg: &[u8]) -> Self {
        let command = CommandResponse::new(msg);

        let sub_type = match command.command.header.message_type() {
            MessageType::QuerySubtype => {
                let body = &command.command.body;

                let t = if body.len() > 2 { body[2] as u16 } else { 0 }
                    + if body.len() > 3 {
                        (body[3] as u16) << 8
                    } else {
                        0
                    };

                t
            }

            _ => 0,
        };

        Self { command, sub_type }
    }
}

impl Deref for CommandSubtypeResponse {
    type Target = CommandResponse;

    fn deref(&self) -> &Self::Target {
        &self.command
    }
}
